#include <avr/pgmspace.h>

// SID Registers
#define VOICE1	0
#define VOICE2	7
#define VOICE3	14
#define CONTROLREG 4
#define ATTACKDECAY 5
#define SUSTAINRELEASE 6

// SID Control Register Bits
#define GATE (1<<0)
#define GATEOFF 0
#define SYNC (1<<1)		// not implemented
#define RINGMOD (1<<2)	
#define TEST (1<<3)		// not implemented
#define TRIANGLE (1<<4)
#define SAWTOOTH (1<<5)
#define RECTANGLE (1<<6)
#define NOISE (1<<7)

// notes VOICE1
#define C1_1	1+VOICE1,0x02,0+VOICE1,0x25,
#define C2_1	1+VOICE1,0x04,0+VOICE1,0x49,
#define C3_1	1+VOICE1,0x08,0+VOICE1,0x93,
#define C4_1	1,0x11,0,0x25,
#define	D4_1	1,0x13,0,0x3F,			
#define	E4_1	1,0x15,0,0x9A,			
#define F4_1	1,0x16,0,0xE3,			
#define	G4_1	1,0x19,0,0x81,			
#define	A4_1	1,0x1C,0,0xC6,			
#define	B4_1	1,0x20,0,0x5E,			
#define	C5_1	1,0x22,0,0x4B,			

// notes VOICE3
#define C2_2	1+VOICE2,0x04,0+VOICE2,0x49,
#define C4_2	1+VOICE2,0x11,0+VOICE2,0x25,
#define	D4_2	1+VOICE2,0x13,0+VOICE2,0x3F,
#define	E4_2	1+VOICE2,0x15,0+VOICE2,0x9A,

// notes VOICE3
#define C2_3	1+VOICE3,0x04,0+VOICE3,0x49,
#define C4_3	1+VOICE3,0x11,0+VOICE3,0x25,
#define	D4_3	1+VOICE3,0x13,0+VOICE3,0x3F,			
#define	E4_3	1+VOICE3,0x15,0+VOICE3,0x9A,			
#define F4_3	1+VOICE3,0x16,0+VOICE3,0xE3,			
#define	G4_3	1+VOICE3,0x19,0+VOICE3,0x81,			
#define	A4_3	1+VOICE3,0x1C,0+VOICE3,0xC6,			
#define	B4_3	1+VOICE3,0x20,0+VOICE3,0x5E,			
#define	C5_3	1+VOICE3,0x22,0+VOICE3,0x4B,

// waveforms
#define SETNOISE_1 		4,0x81,5,0xBB,6,0xAD,
#define SETNOISE_2 		11,0x81,12,0xBB,13,0xAD,
#define SETNOISE_3 		18,0x81,19,0xBB,20,0xAD,
#define SETTRIANGLE_1	4,0x11,5,0xBB,6,0xAA,
#define SETTRIANGLE_2	4+VOICE2,0x11,5+VOICE2,0xBB,6+VOICE2,0xAA,
#define SETTRIANGLE_3	4+VOICE3,0x11,5+VOICE3,0xBB,6+VOICE3,0xAA,
#define SETRECTANGLE_1	VOICE1+CONTROLREG,RECTANGLE+GATE,5+VOICE1,0xBB,6+VOICE1,0xAA,

#define SLOWATTACK 0xB0
#define FASTATTACK 0x00
#define SLOWDECAY 0x0A
#define FASTDECAY 0x00
#define SUSTAINQUITE 0x00
#define SUSTAINNORM	0xA0
#define SUSTAINLOUD 0xF0
#define SLOWRELEASE 0x09
#define FASTRELEASE 0x00


// pause controll, no SID registers, values will be interpreted by the EMULATOR 
#define QUARTER	0xF0,0x00,0xF1,0x01, // pause quarter tone
#define HALF	0xF0,0x00,0xF1,0x02, // pause half tone
#define ONESEC	0xF0,0x00,0xF1,0x04, // pause 2 seconds
#define TWOSEC	0xF0,0x00,0xF1,0x08, // pause 2 seconds
#define FIVESEC	0xF0,0x00,0xF1,40, // pause 2 seconds
#define STOP	0xFF,0xFF, // stops sound generation

/****************************************************************************
	
	static uint8_t sound[]

	Sound Data Array

	The main programm parses the sound data array. 
	It reads the register address and data from the array
	and writes it to the "virtual SID registers".
	0xF0,0xF1 and 0xFF are no SID registers and interpreted as
	controll code for the parser. F0+256*F1 gives the value in milliseconds 
	to pause parsing and let the SID playing it's sound.
	After the pause the next register values are read and written to the SID.
	At the end of the array ther should be the values 0xFF,0xFF which stop 
	the parser and stop sound generation.	 
	
	syntax:

static uint8_t sound[] PROGMEM ={
	sidregister,value,
	sidregister,value,
	....
	0xF0,pauselowvalue,
	0xF1,pausehighvalue,
	....
	sidregister,value,
	sidregister,value,
	0xF0,pauselowvalue,
	0xF1,pausehighvalue,
	0xFF,0xFF			// stop command, turn sound off
};

	April 2007 (c) Christoph Haberer, christoph(at)roboterclub-freiburg.de

****************************************************************************/
static uint8_t sound[] PROGMEM ={

/*************************************************************
			attack,decay,sustain,release example
*************************************************************/


			SETTRIANGLE_1	// set triangle waveform
			
			ATTACKDECAY+VOICE1,SLOWATTACK+SLOWDECAY, 
			SUSTAINRELEASE+VOICE1,SUSTAINNORM+SLOWRELEASE,

			
			//ATTACKDECAY+VOICE1,0xAA, 
			//SUSTAINRELEASE+VOICE1,0x1A,

			//ATTACKDECAY+VOICE1,FASTATTACK+5, 
			//ATTACKDECAY+VOICE1,0xA, 
			//SUSTAINRELEASE+VOICE1,0x8A,
			//SUSTAINRELEASE+VOICE1,SUSTAINQUITE+FASTRELEASE,

			C4_1	// note C4 one channel1 ( voice1 )
			TWOSEC	// wait 2 seconds
			//FIVESEC

			VOICE1+CONTROLREG,TRIANGLE+GATEOFF, // gate off to start release cycle

			TWOSEC	// wait 2 seconds
			
			//STOP

/*************************************************************
			tone latter
*************************************************************/

			SETTRIANGLE_1
			C4_1
			HALF
			D4_1
			HALF
			E4_1
			HALF
			F4_1
			HALF
			G4_1
			HALF
			A4_1
			HALF
			B4_1
			HALF
			C5_1
			HALF

			TWOSEC

/*************************************************************
			3 tone example
*************************************************************/

			SETTRIANGLE_1
			C4_1
			HALF
			SETTRIANGLE_2
			E4_2
			HALF
			SETTRIANGLE_3
			G4_3
			HALF

			TWOSEC
			
			// start release cycle
			VOICE1+CONTROLREG,TRIANGLE+GATEOFF,
			VOICE2+CONTROLREG,TRIANGLE+GATEOFF,
			VOICE3+CONTROLREG,TRIANGLE+GATEOFF,

			TWOSEC
			TWOSEC

/*************************************************************
			PWM example
*************************************************************/
			
			SETRECTANGLE_1
			C4_1

			2,0x00,	// set PWM value
			3,0x04,	// set PWM value
			ONESEC
			
			2,0x00,	// set PWM value
			3,0x06,	// set PWM value
			ONESEC

			2,0x00,	// set PWM value
			3,0x08,	// set PWM value
			TWOSEC

/*************************************************************
			ring modulation example
*************************************************************/

			SETTRIANGLE_1
			F4_1

			6,0x8B, // sustain/release

			SETTRIANGLE_3
			C4_3
			6+VOICE3,0x08, // sustain/release

			VOICE1+CONTROLREG,0x14,	// set ringmod, gate off for decay
			VOICE3+CONTROLREG,0x00,	// gate off
			
			TWOSEC
			TWOSEC

/*************************************************************
			noise example
*************************************************************/

			// shotnoise
			1,0x40,
			0,0x00,
			SETNOISE_1
			ATTACKDECAY+VOICE1,FASTATTACK+5, 
			SUSTAINRELEASE+VOICE1,SUSTAINQUITE+FASTRELEASE,
			QUARTER

			// shotnoise
			1,0x40,
			0,0x00,
			SETNOISE_1
			ATTACKDECAY+VOICE1,FASTATTACK+5, 
			SUSTAINRELEASE+VOICE1,SUSTAINQUITE+FASTRELEASE,
			QUARTER

			// shotnoise
			1,0x40,
			0,0x00,
			SETNOISE_1
			ATTACKDECAY+VOICE1,FASTATTACK+5, 
			SUSTAINRELEASE+VOICE1,SUSTAINQUITE+FASTRELEASE,
			QUARTER
			
			VOICE1+CONTROLREG,NOISE+GATEOFF,

			TWOSEC

			// light noise
			SETNOISE_1
			1,0x40,
			0,0x00,	
			
			ATTACKDECAY+VOICE1,SLOWATTACK+SLOWDECAY, 
			SUSTAINRELEASE+VOICE1,SUSTAINNORM+0x0B,
			TWOSEC

			VOICE1+CONTROLREG,NOISE+GATEOFF,
			TWOSEC

			// dark noise
			SETNOISE_1
			1,0x05,
			0,0x00,		
			TWOSEC

/*************************************************************
			3 tone example
*************************************************************/

			SETTRIANGLE_1
			C4_1
			HALF
			SETTRIANGLE_2
			E4_2
			HALF
			SETTRIANGLE_3
			G4_3
			HALF

			TWOSEC
			
			// start release cycle
			VOICE1+CONTROLREG,TRIANGLE+GATEOFF,
			VOICE2+CONTROLREG,TRIANGLE+GATEOFF,
			VOICE3+CONTROLREG,TRIANGLE+GATEOFF,

			TWOSEC
			TWOSEC


/*************************************************************
			3 tone gong
*************************************************************/
/*
			SETTRIANGLE_1
			ATTACKDECAY+VOICE1,FASTATTACK+SLOWDECAY, 
			SUSTAINRELEASE+VOICE1,1+SLOWRELEASE,
			C4_1
			HALF

			SETTRIANGLE_2
			ATTACKDECAY+VOICE2,FASTATTACK+SLOWDECAY, 
			SUSTAINRELEASE+VOICE2,1+SLOWRELEASE,
			D4_2
			HALF

			SETTRIANGLE_3
			ATTACKDECAY+VOICE3,FASTATTACK+SLOWDECAY, 
			SUSTAINRELEASE+VOICE3,1+SLOWRELEASE,
			E4_3
			HALF

			TWOSEC

			// start release cycle
			VOICE1+CONTROLREG,TRIANGLE+GATEOFF,
			VOICE2+CONTROLREG,TRIANGLE+GATEOFF,
			VOICE3+CONTROLREG,TRIANGLE+GATEOFF,
*/			
/*************************************************************/
			STOP
	};

